<?php
	/*
	 * beroCloudAPI Class to exchange with beroCloud
	 * Must be extended by SbcApi or CloudSbcApi
	 */
if (!function_exists('curl_init')) {
	## class can not be used
	exit(0);
}

include_once('/usr/php/include/rootHelper.class.php');
class beroCloudAPI {

	public	$has_error		= false,
					$is_init			= false,
					$socket				= null,
					$socket_error	= null;

	public	$key				= '',
					$proxy_data	= '',
					$url				= '',
					$version 		= '';

	public	$log_file	= '',
					$serial		= '';

	public $bc = null;

	/* [CONSTRUCTOR] */
	function __construct($bc = null) {
		$this->version = '3.0';
		if (is_null($bc)) {
			require_once('/usr/fallback/beroConf.php');
			$bc = new beroConf('root');
		}
		$this->bc = $bc;
	}

	/* [PUBLIC METHODS] */
	public function hasError() {
		return($this->has_error);
	}

	public function getError() {
		return($this->socket_error);
	}

	public function getSerial() {
		return($this->serial);
	}

	public function log($error) {
		file_put_contents($this->log_file, rootHelper::getDate() ." $error". PHP_EOL, FILE_APPEND);
		if (filesize($this->log_file) > 3000000) { // aka > 3MB
			copy($this->log_file, "{$this->log_file}.1");
			@unlink($this->log_file);
		}
	}

	public function sendRequest($method, $uri, $data = null, $options = array(), &$info = array()) {
		if ($this->has_error) {
			return false;
		}
		$default_options = array(
			'asJSON'				=> 0,
			'cipherlist'		=> 'TLSv1.2',
			'connecttimeout'=> 100,
			'debug'					=> 0,
			'handle'				=> null,
			'returntransfer'=> 1,
			'to-berocloud'	=> true,
			'timeout'				=> 100,
			'verifyhost'		=> 2,
			'verifypeer'		=> 1,
		);
		$options = array_merge($default_options, $options);
		try {
			if (!$this->is_init) {
				throw new Exception('URL information missing');
			}

			$ch = curl_init();
			if ($ch === false) {
				throw new Exception('Failed to initialize cURL');
			}

			// set basic options
			curl_setopt($ch, CURLOPT_URL, $options['to-berocloud'] ? "{$this->url}?r=$uri" : "{$this->url}$uri");
			curl_setopt($ch, CURLOPT_RETURNTRANSFER, (int) $options['returntransfer']);
			curl_setopt($ch, CURLOPT_CONNECTTIMEOUT, (int) $options['connecttimeout']);
			curl_setopt($ch, CURLOPT_TIMEOUT, (int) $options['timeout']);

			// set CA info + ssl connection
			foreach (array('debug', 'verifypeer', 'verifyhost') as $what) {
				$val = $this->bc->get('root', "openssl_$what");
				if (strlen($val)) {
					$options[$what] = $val;
				}
			}

			$tlsPath = '/usr/conf/tls';
			if (file_exists("{$tlsPath}/certs/ca-certs.crt")) {
				if ($options['verifypeer'] == 1) {
					curl_setopt($ch, CURLOPT_CAINFO, "{$tlsPath}/certs/ca-certs.crt");
				}
			}
			else {
				$options['verifyhost'] = 0;
				$options['verifypeer'] = 0;
				$this->bc->set('root', 'openssl-ca-missing', 'OPENSSL_CACRT_MISSING');
			}

			curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, (int) $options['verifyhost']);
			curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, (int) $options['verifypeer']);

			if (strlen($options['ssl-cipher-list']) > 0) {
				curl_setopt($ch, CURLOPT_SSL_CIPHER_LIST, $options['ssl-cipher-list']);
			}

			// file handling
			if (!is_null($options['handle'])) {
				curl_setopt($ch, CURLOPT_FILE, $options['handle']);
			}

			// data
			if ($options['to-berocloud']) {
				$data = array_merge(array('key' => $this->key, 'serial' => $this->serial), $data);
			}

			// method
			switch ($method) {
			case 'delete':
				curl_setopt($ch, CURLOPT_CUSTOMREQUEST, "DELETE");
				break;
			case 'get':
				break;
			case 'post':
				curl_setopt($ch, CURLOPT_POST, 1);
				break;
			case 'put':
				curl_setopt($ch, CURLOPT_CUSTOMREQUEST, "PUT");
				break;
			default:
				throw new Exception("Unknown method=$method. Only 'delete', 'get', 'post' and 'put' accepted");
			}

			// proxy
			if ((strlen($this->proxy_data['Address']) != 0) && (strlen($this->proxy_data['Port']) != 0) && (strlen($this->proxy_data['Type']) != 0)) {
				curl_setopt($ch, CURLOPT_PROXY, $this->proxy_data['Address']);
				curl_setopt($ch, CURLOPT_PROXYPORT, $this->proxy_data['Port']);
				curl_setopt($ch, CURLOPT_PROXYTYPE, $this->proxy_data['Type']);
				if (strlen($this->proxy_data['User']) != 0 && strlen($this->proxy_data['Secret']) != 0) {
					curl_setopt($ch, CURLOPT_PROXYUSERPWD, "{$this->proxy_data['User']}:{$this->proxy_data['Secret']}");
				}
			}

			// as JSON
			if (!empty($data)) {
				if ($options['asJSON']) {
					curl_setopt($ch, CURLOPT_HTTPHEADER, array('Content-Type:application/json'));
					curl_setopt($ch, CURLOPT_POSTFIELDS, rootHelper::jsonEncode($data));
				}
				else {
					curl_setopt($ch, CURLOPT_POSTFIELDS, $data);
				}
			}

			// debug
			$fp = null;
			if ($options['debug']) {
				curl_setopt($ch, CURLOPT_VERBOSE, 1);
				if ($fp = fopen('/tmp/curl.cloud.log', 'w')) {
					curl_setopt($ch, CURLOPT_STDERR, $fp);
				}
			}

			// exec
			$response = curl_exec($ch);
			$info = curl_getinfo($ch);

			// fetch error
			$errno = curl_errno($ch);
			$error = curl_error($ch);

			// close curl
			curl_close($ch);

			if (!is_null($fp)) {
				fclose($fp);
			}

			// debug
			if ($options['debug']) {
				$this->log("###### DEBUG #####\nmethod=$method|uri=$uri|error=$error|errno=$errno|". print_r($data, true).print_r($options, true).print_r($info, true) ."\n###########");
			}

			// manage response
			if ($response === false) {
				$this->log("cURL error: [errno=$errno|error=$error]");
				throw new Exception($error, $errno);
			}

			if (!in_array($info['http_code'], array('200', '201'))) {
				$this->log("HTTP error: [code={$info['http_code']}]");
				return(false);
			}

			if ($response == false) {
				return(true);
			}
			return($options['asJSON'] ? rootHelper::jsonDecode($response, true) : $response);
		}
		catch (Exception $e) {
			$this->has_error = true;
			$this->socket_error = $this->_setError($e);
			return false;
		}
	}

	/* [PRIVATE METHODS] */
	private function _setError($exception) {
		return array(
			'code'	=> $exception->getCode(),
			'file'	=> $exception->getFile(),
			'line'	=> $exception->getLine(),
			'msg'		=> $exception->getMessage(),
		);
	}
}
?>
